// INTEGRATION OF ODEs
// This is an example of computation of Poincare maps in CAPD library
// Below we have two applications: 
// 1. non-rigorous computations in "PoincareNonRigorous"
// 2. interval-rigorous computations in "PoincareInterval"

#include <iostream>
using namespace std;

#include "capd/capdlib.h"
using namespace capd;
using namespace capd::alglib;
using namespace capd::matrixAlgorithms;

// 1.
void PoincareNonRigorous()
{
	cout << endl << "Non-rigorous computations: " << endl << endl;

	// DECLARING A POINCARE MAP FOR A VECTOR FIELD f AND A SECTION {s=0}
   	int order = 20;
   	DMap f = "var:x,y,z;fun:x*(1-(x^2+y^2)^(-0.5))-y,x+y*(1-(x^2+y^2)^(-0.5)),-z;";
   	DCoordinateSection section(3,0); 
   	DOdeSolver solver(f,order);
   	DPoincareMap P(solver,section);
	
	// IMAGE OF A DVector BY THE POINCARE MAP
	DVector x(3);
	x[0] = 0;   
	x[1] = 1;   
	x[2] = 0.5;
	DVector y(3);	

	y = P(x);
	cout << "P(x) = " << y << endl;

	// IMAGE OF A DVector TOGETHER WITH THE DERIVATIVE OF THE POINCARE MAP
	DMatrix DPhi(3,3);
	DMatrix DP(3,3);

	y = P(x,DPhi);

	DP = P.computeDP(y,DPhi);

	cout << "P(x) = " << y << endl;
	cout << "DP(x) = " << DP << endl << endl;
}
// 2.
void PoincareInterval()
{
	cout << endl << "Interval computations: " << endl << endl;

	// DECLARING A POINCARE MAP FOR A VECTOR FIELD f AND A SECTION {s=0}
   	int order = 20;
   	IMap f = "var:x,y,z;fun:x*(1-(x^2+y^2)^(-0.5))-y,x+y*(1-(x^2+y^2)^(-0.5)),-z;";
   	ICoordinateSection section(3,0);
	IOdeSolver T(f,order);
   	IPoincareMap P(T,section);

	// IMAGE OF AN IVector BY THE POINCARE MAP
	IVector x(3);
	x[0] = interval(0);
	x[1] = interval(1);
	x[2] = interval(1)/interval(2);
	IVector y(3);

	C0Rect2Set R(x);
	
	y = P(R);

	cout << "P(x) = " << y << endl << endl;
	
	// IMAGE OF AN IVector TOGETHER WITH THE DERIVATIVE OF THE POINCARE MAP
	IMatrix DPhi(3,3);
	IMatrix DP(3,3);
	C1Rect2Set S(x);
	
	y = P(S,DPhi);
	DP = P.computeDP(y,DPhi);

	cout << "P(x) = " << y << endl;
	cout << "DP(x) = " << DP << endl << endl;
}
int main(int argc, char* argv[])
{
  	cout.precision(6);
	try
	{
		PoincareNonRigorous();
		PoincareInterval();				
	}
	catch(exception& e)
  	{
    		cout << "\n\nException caught: "<< e.what() << endl;
  	}
  return 0;
} 
