// INTEGRATION OF ODEs
// This is an example of computation of time shift maps for ODEs in CAPD library
// Below we have two applications: 
// 1. non-rigorous computations in "NonRigorous"
// 2. interval-rigorous computations in "IntervalRigorous"

#include <iostream>
using namespace std;

#include "capd/capdlib.h"
using namespace capd;
using namespace capd::alglib;
using namespace capd::matrixAlgorithms;
using SolutionCurve = capd::ITimeMap::SolutionCurve;

// 1.
void NonRigorous()
{
	cout << endl << "Non-rigorous computations: " << endl << endl;
	// DECLARING A TIME SHIFT MAP FOR A VECTOR FIELD f
   	int order = 20;
	DMap f = "var:x,y;fun:x*(1-(x^2+y^2)^(-0.5))-y,x+y*(1-(x^2+y^2)^(-0.5));";
	DOdeSolver solver(f,order);
	DTimeMap Phi(solver);
	double T =3.14159265358979;
		
	// IMAGE OF A DVector BY THE TIME SHIFT MAP	
	DVector x(2);
	x[0] = 1.;
	x[1] = 0.;
	DVector y(2);

	y = Phi(T,x);
	cout << "Phi(x) = " << y << endl << endl;

	// IMAGE OF A DVector TOGETHER WITH THE DERIVATIVE
	DMatrix der(2,2);

	y = Phi(T,x,der);
	
	cout << "Phi(x) = " << y << endl;
	cout << "DPhi(x) = " << der << endl;
	
}
// 2.
void IntervalRigorous()
{
	cout << endl << "Interval computations: " << endl << endl;

	// DECLARING A TIME SHIFT MAP FOR A VECTOR FIELD
   	int order = 20;
	IMap f = "var:x,y;fun:x*(1-(x^2+y^2)^(-0.5))-y,x+y*(1-(x^2+y^2)^(-0.5));";
	IOdeSolver solver(f,order);
	ITimeMap Phi(solver);
	interval T = 2*4*atan(interval(1));

	// IMAGE OF AN IVector BY THE TIME SHIFT MAP	
	IVector x(2);
	x[0] = interval(1);
	x[1] = interval(0);
	C0Rect2Set R(x);
	IVector y(2);
	
	y = Phi(T,R);

	cout << "Phi(x) = " << y << endl << endl;

	// IMAGE OF AN IVector TOGETHER WITH THE DERIVATIVE
	IMatrix der(2,2);
	C1Rect2Set S(x);

	y = Phi(T,S,der);
	
	cout << "Phi(x) = " << y << endl;
	cout << "DPhi(x) = " << der << endl << endl;
	
	SolutionCurve curve(0.0);
	S = C1Rect2Set(x);
	Phi(T,S,curve);
	cout << "for t=[1.0,1.1] we have " << endl;
	cout << "Phi(t,x) = " << curve(interval(1.0,1.1)) << endl;
	
}

int main(int argc, char* argv[])
{
  	cout.precision(6);
	try
	{
		NonRigorous();
		IntervalRigorous();				
	}
	catch(exception& e)
  	{
    		cout << "\n\nException caught: "<< e.what() << endl;
  	}
  return 0;
} 
